@@ -2,7 +2,7 @@
  *
  *  Wireless daemon for Linux
  *
- *  Copyright (C) 2018  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2018-2019  Intel Corporation. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -20,8 +20,8 @@
  *
  */
 
-const char *properties_on_off_opts[3];
-const char *properties_yes_no_opts[3];
+extern const char *properties_on_off_opts[3];
+extern const char *properties_yes_no_opts[3];
 
 bool properties_builder_append_on_off_variant(
 					struct l_dbus_message_builder *builder,
