@@ -1168,14 +1168,14 @@ LIB_EXPORT bool l_dhcp_server_set_ip_range(struct l_dhcp_server *server,
 
 LIB_EXPORT bool l_dhcp_server_set_debug(struct l_dhcp_server *server,
 				l_dhcp_debug_cb_t function,
-				void *user_data, l_dhcp_destroy_cb_t destory)
+				void *user_data, l_dhcp_destroy_cb_t destroy)
 {
 	if (unlikely(!server))
 		return false;
 
 	server->debug_handler = function;
 	server->debug_data = user_data;
-	server->debug_destroy = destory;
+	server->debug_destroy = destroy;
 
 	return true;
 }
