@@ -1,37 +1,23 @@
 /*
+ * Embedded Linux library
+ * Copyright (C) 2016  Intel Corporation
  *
- *  Embedded Linux library
- *
- *  Copyright (C) 2016  Intel Corporation. All rights reserved.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #ifndef __ELL_KEY_H
 #define __ELL_KEY_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #include <stddef.h>
 #include <stdbool.h>
 
+#include <ell/cleanup.h>
 #include <ell/checksum.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct l_key;
 struct l_keyring;
 
@@ -44,6 +30,7 @@ enum l_key_feature {
 enum l_key_type {
 	L_KEY_RAW = 0,
 	L_KEY_RSA,
+	L_KEY_ECC,
 };
 
 enum l_keyring_restriction {
@@ -54,6 +41,7 @@ enum l_keyring_restriction {
 enum l_key_cipher_type {
 	L_KEY_RSA_PKCS1_V1_5,
 	L_KEY_RSA_RAW,
+	L_KEY_ECDSA_X962,
 };
 
 struct l_key *l_key_new(enum l_key_type type, const void *payload,
@@ -108,7 +96,9 @@ bool l_keyring_restrict(struct l_keyring *keyring, enum l_keyring_restriction re
 			const struct l_keyring *trust);
 
 void l_keyring_free(struct l_keyring *keyring);
+DEFINE_CLEANUP_FUNC(l_keyring_free);
 void l_keyring_free_norevoke(struct l_keyring *keyring);
+DEFINE_CLEANUP_FUNC(l_keyring_free_norevoke);
 
 bool l_keyring_link(struct l_keyring *keyring, const struct l_key *key);
 
