@@ -45,6 +45,7 @@
 #include "ell/useful.h"
 
 #include "src/missing.h"
+#include "src/defs.h"
 #include "src/common.h"
 #include "src/storage.h"
 #include "src/crypto.h"
@@ -53,6 +54,7 @@
 #define STORAGE_FILE_MODE (S_IRUSR | S_IWUSR)
 
 #define KNOWN_FREQ_FILENAME ".known_network.freq"
+#define EAP_TLS_CACHE_FILENAME ".eap-tls-session-cache"
 
 static char *storage_path = NULL;
 static char *storage_hotspot_path = NULL;
@@ -314,15 +316,19 @@ const char *storage_network_ssid_from_path(const char *path,
 		return NULL;
 
 	if (filename[0] != '=') {
-		if (end == filename || end - filename > 32)
+		if (end == filename || end - filename > SSID_MAX_SIZE)
 			return NULL;
 
 		for (c = filename; c < end; c++)
 			if (!isalnum(*c) && !strchr("-_ ", *c))
 				break;
 
-		if (c < end)
+		if (c < end) {
+			l_warn("Provisioning file %s contains non-alphanumeric "
+				"characters in the name. Please hex-encode. "
+				"See man iwd.network", path);
 			return NULL;
+		}
 
 		memcpy(buf, filename, end - filename);
 		buf[end - filename] = '\0';
@@ -456,11 +462,9 @@ char *__storage_encrypt(const struct l_settings *settings, const char *name,
 int __storage_decrypt(struct l_settings *settings, const char *ssid,
 				bool *encrypt)
 {
-	_auto_(l_settings_free) struct l_settings *security = NULL;
 	_auto_(l_free) uint8_t *encrypted = NULL;
 	_auto_(l_free) uint8_t *decrypted = NULL;
 	_auto_(l_free) uint8_t *salt = NULL;
-	_auto_(l_strv_free) char **embedded = NULL;
 	_auto_(l_strv_free) char **groups = NULL;
 	char **i;
 	size_t elen, slen;
@@ -496,6 +500,13 @@ int __storage_decrypt(struct l_settings *settings, const char *ssid,
 		return 0;
 	}
 
+	/*
+	 * It should likely be far larger than this, but that will get caught
+	 * later when reloading the decrypted data.
+	 */
+	if (elen < 16)
+		return -EBADMSG;
+
 	/*
 	 * AES-SIV automatically verifies the IV (16 bytes) and returns only
 	 * the decrypted data portion. We add one here for the NULL terminator
@@ -531,7 +542,7 @@ int __storage_decrypt(struct l_settings *settings, const char *ssid,
 
 	/*
 	 * Load decrypted data into existing settings. This is not how the API
-	 * is indended to be used (since this could result in duplicate groups)
+	 * is intended to be used (since this could result in duplicate groups)
 	 * but since the Security group was just removed and EncryptedSecurity
 	 * should only contain a Security group its safe to use it this way.
 	 */
@@ -592,15 +603,17 @@ struct l_settings *storage_network_open(enum security type, const char *ssid)
 	struct l_settings *settings;
 	_auto_(l_free) char *path = NULL;
 
-	if (ssid == NULL)
+	if (!ssid)
 		return NULL;
 
 	path = storage_get_network_file_path(type, ssid);
 
 	settings = l_settings_new();
 
-	if (!l_settings_load_from_file(settings, path))
+	if (!l_settings_load_from_file(settings, path)) {
+		l_error("Error loading %s", path);
 		goto error;
+	}
 
 	if (type != SECURITY_NONE && !storage_decrypt(settings, path, ssid))
 		goto error;
@@ -617,7 +630,7 @@ int storage_network_touch(enum security type, const char *ssid)
 	char *path;
 	int ret;
 
-	if (ssid == NULL)
+	if (!ssid)
 		return -EINVAL;
 
 	path = storage_get_network_file_path(type, ssid);
@@ -697,6 +710,47 @@ void storage_known_frequencies_sync(struct l_settings *known_freqs)
 	l_free(known_freq_file_path);
 }
 
+struct l_settings *storage_eap_tls_cache_load(void)
+{
+	_auto_(l_free) char *path =
+		storage_get_path("%s", EAP_TLS_CACHE_FILENAME);
+	struct l_settings *cache = l_settings_new();
+
+	if (!l_settings_load_from_file(cache, path))
+		l_debug("No session cache loaded from %s, starting with an "
+			"empty cache", path);
+
+	return cache;
+}
+
+void storage_eap_tls_cache_sync(const struct l_settings *cache)
+{
+	_auto_(l_free) char *path =
+		storage_get_path("%s", EAP_TLS_CACHE_FILENAME);
+	_auto_(l_free) char *settings_data = NULL;
+	_auto_(l_free) char *data = NULL;
+	size_t len;
+	static const char comment[] =
+		"# External changes to this file are not tracked by IWD "
+		"and will be overwritten.\n\n";
+	static const size_t comment_len = L_ARRAY_SIZE(comment) - 1;
+
+	settings_data = l_settings_to_data(cache, &len);
+	data = l_malloc(comment_len + len);
+	memcpy(data, comment, comment_len);
+	memcpy(data + comment_len, settings_data, len);
+
+	/*
+	 * Note this data contains cryptographic secrets.  write_file()
+	 * happens to set the right permissions on the file.
+	 *
+	 * TODO: consider encrypting with system_key.
+	 */
+	write_file(data, comment_len + len, false, "%s", path);
+	explicit_bzero(settings_data, len);
+	explicit_bzero(data, len);
+}
+
 bool storage_is_file(const char *filename)
 {
 	char *path;
