@@ -500,7 +500,7 @@ static bool extract_ssid(struct wsc_attr_iter *iter, void *data)
 	unsigned int len;
 
 	len = wsc_attr_iter_get_length(iter);
-	if (len > 32)
+	if (len > SSID_MAX_SIZE)
 		return false;
 
 	ssid->iov_len = len;
@@ -722,7 +722,7 @@ int wsc_parse_attrs(const unsigned char *pdu, unsigned int len,
 			}
 		}
 
-		if (e2 == NULL) {
+		if (!e2) {
 			if (!ext_iter)
 				break;
 
