@@ -81,18 +81,20 @@ static const struct hmac_data test_case_2 = {
 	.hmac		= "de7c9b85b8b78aa6bc8a7a36f70a90701c9db4d9",
 };
 
+static bool test_precheck(const void *data)
+{
+	return l_checksum_is_supported(L_CHECKSUM_SHA1, true);
+}
+
+#define add_test(name, func, data) l_test_add_data_func_precheck(name, data, \
+							func, test_precheck, 0)
+
 int main(int argc, char *argv[])
 {
 	l_test_init(&argc, &argv);
 
-	if (!l_checksum_is_supported(L_CHECKSUM_SHA1, true)) {
-		printf("SHA1 support missing, skipping...\n");
-		goto done;
-	}
+	add_test("/hmac-sha1/Test case 1", hmac_test, &test_case_1);
+	add_test("/hmac-sha1/Test case 2", hmac_test, &test_case_2);
 
-	l_test_add("/hmac-sha1/Test case 1", hmac_test, &test_case_1);
-	l_test_add("/hmac-sha1/Test case 2", hmac_test, &test_case_2);
-
-done:
 	return l_test_run();
 }
