@@ -238,10 +238,15 @@ static void rx_packet(struct ethdev *dev, const uint8_t *addr,
 
 			l_debug("Created new EAPoL session");
 
-			l_queue_push_tail(dev->eapol_sessions, eapol);
-
 			eapol->cred = network_lookup_security("default");
-			eap_load_settings(eapol->eap, eapol->cred, "EAP-");
+			if (!eapol->cred || !eap_load_settings(eapol->eap,
+						eapol->cred, "EAP-")) {
+				l_error("Failed to load EAP settings");
+				eapol_free(eapol);
+				return;
+			}
+
+			l_queue_push_tail(dev->eapol_sessions, eapol);
 
 			eap_set_key_material_func(eapol->eap, eap_key_material);
 			eap_set_event_func(eapol->eap, eap_event);
@@ -307,8 +312,9 @@ static bool pae_read(struct l_io *io, void *user_data)
 	return true;
 }
 
-static void pae_destroy()
+static void pae_destroy(void * user_data)
 {
+	(void)user_data;
 	pae_io = NULL;
 }
 
@@ -386,7 +392,7 @@ static char *read_devtype_from_uevent(const char *ifname)
 	return devtype;
 }
 
-static int modify_membership(struct ethdev *dev, int optname)
+static int modify_membership(uint32_t index, int optname)
 {
 	struct packet_mreq mreq;
 	int fd;
@@ -396,7 +402,7 @@ static int modify_membership(struct ethdev *dev, int optname)
 		return -1;
 
 	memset(&mreq, 0, sizeof(mreq));
-	mreq.mr_ifindex = dev->index;
+	mreq.mr_ifindex = index;
 	mreq.mr_type = PACKET_MR_MULTICAST;
 	mreq.mr_alen = ETH_ALEN;
 	memcpy(mreq.mr_address, pae_group_addr, ETH_ALEN);
@@ -410,7 +416,8 @@ static void ethdev_free(void *data)
 
 	l_debug("Freeing device %s", dev->ifname);
 
-	modify_membership(dev, PACKET_DROP_MEMBERSHIP);
+	if (modify_membership(dev->index, PACKET_DROP_MEMBERSHIP) < 0)
+		l_error("Failed to drop membership");
 
 	l_queue_destroy(dev->eapol_sessions, eapol_free);
 
@@ -516,6 +523,11 @@ static void newlink_notify(const struct ifinfomsg *ifi, int bytes)
 			}
 		}
 
+		if (modify_membership(index, PACKET_ADD_MEMBERSHIP) < 0) {
+			l_error("Failed to add membership");
+			return;
+		}
+
 		dev = l_new(struct ethdev, 1);
 		dev->index = index;
 		dev->active = active;
@@ -527,8 +539,6 @@ static void newlink_notify(const struct ifinfomsg *ifi, int bytes)
 
 		l_debug("Creating device %u", dev->index);
 
-		modify_membership(dev, PACKET_ADD_MEMBERSHIP);
-
 		l_dbus_object_add_interface(dbus_app_get(), dev->path,
 						ADAPTER_INTERFACE, dev);
 
@@ -714,7 +724,8 @@ static void setup_adapter_interface(struct l_dbus_interface *interface)
 
 bool ethdev_init(const char *whitelist, const char *blacklist)
 {
-	struct ifinfomsg msg;
+	struct ifinfomsg ifi;
+	struct l_netlink_message *nlm;
 
 	if (rtnl)
 		return false;
@@ -751,10 +762,11 @@ bool ethdev_init(const char *whitelist, const char *blacklist)
 	if (blacklist)
 		blacklist_filter = l_strsplit(blacklist, ',');
 
-	memset(&msg, 0, sizeof(msg));
+	memset(&ifi, 0, sizeof(ifi));
+	nlm = l_netlink_message_new_sized(RTM_GETLINK, NLM_F_DUMP, sizeof(ifi));
+	l_netlink_message_add_header(nlm, &ifi, sizeof(ifi));
 
-	l_netlink_send(rtnl, RTM_GETLINK, NLM_F_DUMP, &msg, sizeof(msg),
-						getlink_callback, NULL, NULL);
+	l_netlink_send(rtnl, nlm, getlink_callback, NULL, NULL);
 
 	return true;
 }
